@@ -20,10 +20,9 @@
 #include <nativehelper/JNIHelp.h>
 
 #include "android_os_Parcel.h"
-#include "android/graphics/GraphicsJNI.h"
-#include "android/graphics/GraphicBuffer.h"
 
 #include <android/hardware_buffer.h>
+#include <android_runtime/android_graphics_GraphicBuffer.h>
 #include <android_runtime/android_hardware_HardwareBuffer.h>
 #include <android_runtime/AndroidRuntime.h>
 #include <android_runtime/Log.h>
@@ -31,8 +30,8 @@
 
 #include <binder/Parcel.h>
 
-#include <ui/GraphicBuffer.h>
 #include <private/gui/ComposerService.h>
+#include <ui/PixelFormat.h>
 
 #include <hardware/gralloc1.h>
 #include <grallocusage/GrallocUsageConversion.h>
@@ -86,7 +85,7 @@ static jlong android_hardware_HardwareBuffer_create(JNIEnv* env, jobject clazz,
     sp<GraphicBuffer> buffer = new GraphicBuffer(width, height, pixelFormat, layers,
             grallocUsage, std::string("HardwareBuffer pid [") + std::to_string(getpid()) +"]");
     status_t error = buffer->initCheck();
-    if (error < 0) {
+    if (error != OK) {
         if (kDebugGraphicBuffer) {
             ALOGW("createGraphicBuffer() failed in HardwareBuffer.create()");
         }
@@ -98,7 +97,8 @@ static jlong android_hardware_HardwareBuffer_create(JNIEnv* env, jobject clazz,
 }
 
 static jlong android_hardware_HardwareBuffer_createFromGraphicBuffer(JNIEnv* env, jobject clazz, jobject graphicBuffer) {
-    sp<GraphicBuffer> buffer(graphicBufferForJavaObject(env, graphicBuffer));
+    sp<GraphicBuffer> buffer(android_graphics_GraphicBuffer_getNativeGraphicsBuffer(env,
+                                                                                    graphicBuffer));
     GraphicBufferWrapper* wrapper = new GraphicBufferWrapper(buffer);
     return reinterpret_cast<jlong>(wrapper);
 }
@@ -166,6 +166,20 @@ static jlong android_hardware_HardwareBuffer_getUsage(JNIEnv* env,
     return AHardwareBuffer_convertFromGrallocUsageBits(buffer->getUsage());
 }
 
+static jlong android_hardware_HardwareBuffer_estimateSize(jlong nativeObject) {
+    GraphicBuffer* buffer = GraphicBufferWrapper_to_GraphicBuffer(nativeObject);
+
+    uint32_t bpp = bytesPerPixel(buffer->getPixelFormat());
+    if (bpp == 0) {
+        // If the pixel format is not recognized, use 1 as default.
+        bpp = 1;
+    }
+
+    const uint32_t bufferStride =
+            buffer->getStride() > 0 ? buffer->getStride() : buffer->getWidth();
+    return static_cast<jlong>(buffer->getHeight() * bufferStride * bpp);
+}
+
 // ----------------------------------------------------------------------------
 // Serialization
 // ----------------------------------------------------------------------------
@@ -209,6 +223,16 @@ AHardwareBuffer* android_hardware_HardwareBuffer_getNativeHardwareBuffer(
     }
 }
 
+GraphicBuffer* android_hardware_HardwareBuffer_getNativeGraphicBuffer(
+        JNIEnv* env, jobject hardwareBufferObj) {
+    if (env->IsInstanceOf(hardwareBufferObj, gHardwareBufferClassInfo.clazz)) {
+        return GraphicBufferWrapper_to_GraphicBuffer(
+                env->GetLongField(hardwareBufferObj, gHardwareBufferClassInfo.mNativeObject));
+    } else {
+        return nullptr;
+    }
+}
+
 jobject android_hardware_HardwareBuffer_createFromAHardwareBuffer(
         JNIEnv* env, AHardwareBuffer* hardwareBuffer) {
     GraphicBuffer* buffer = AHardwareBuffer_to_GraphicBuffer(hardwareBuffer);
@@ -247,6 +271,7 @@ uint64_t android_hardware_HardwareBuffer_convertToGrallocUsageBits(uint64_t usag
 
 const char* const kClassPathName = "android/hardware/HardwareBuffer";
 
+// clang-format off
 static const JNINativeMethod gMethods[] = {
     { "nCreateHardwareBuffer",  "(IIIIJ)J",
             (void*) android_hardware_HardwareBuffer_create },
@@ -267,7 +292,11 @@ static const JNINativeMethod gMethods[] = {
     { "nGetFormat", "(J)I",     (void*) android_hardware_HardwareBuffer_getFormat },
     { "nGetLayers", "(J)I",     (void*) android_hardware_HardwareBuffer_getLayers },
     { "nGetUsage", "(J)J",      (void*) android_hardware_HardwareBuffer_getUsage },
+
+    // --------------- @CriticalNative ----------------------
+    { "nEstimateSize", "(J)J",  (void*) android_hardware_HardwareBuffer_estimateSize },
 };
+// clang-format on
 
 int register_android_hardware_HardwareBuffer(JNIEnv* env) {
     int err = RegisterMethodsOrDie(env, kClassPathName, gMethods,
